/*

 Coded Instinct 1.0

 Copyright (C) 2007 Jose Francisco Castro <francisco@imov.com.ar>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 02110-1301, USA.

*/

#include <io.h>
#include <wdt.h>
#include <stdlib.h>

#define MIN_PWM 8
#define MAX_PWM 230     // safe:230
#define DELTA 1         // safe:4

#define pwm0	OCR1A
#define pwm1	OCR1B


void init_pwm()
{
    DDRB	= 0x0A;	// set portb1 and portb3 as outputs
    PORTB	= 0x00;
    PLLCSR	= 0x00;	// set timer1 clock to system clock
    TCCR1A	= 0xA3;	// configure OCR1A and OCR1B for PWM
    TCCR1B	= 0x84;	// configure CK/1 as clock source
    OCR1C	= 0xFF;	// set max count for time1 to 255
}


void delay(unsigned char time)
{
    TCNT0 = 0xFF-time;
    TIFR = 1<<TOV0;
    while(!(TIFR & 1<<TOV0));
}


void init_adc()
{
    DDRA &= 0x01;
    ADCSR = 1<<ADEN | 0<<ADPS2 | 1<<ADPS1 | 1<<ADPS0;
    ADMUX = 1<<ADLAR;				// set to right-adjusted result, vref AVCC, ch 0
}

unsigned char read_adc()
{
    while(TCNT1!=253);
    ADCSR |= (1<<ADSC); 
    while(ADCSR & (1<<ADSC));       // wait until conversion complete
    return ADCH;    // read ADC (low 8 bits);
}

                                                                                                                                    
unsigned char light_change()                                                                                                        
{                                                                                                                                   
    unsigned char static adc;                                                                                                       
    unsigned char static old_adc;                                                                                                   

    old_adc = (old_adc + adc) >> 1;
    adc = read_adc();

    if(abs(old_adc-adc) > DELTA)
        return abs(old_adc - adc);
    else
        return 0;
}

int main(void) 
{
    unsigned char delay_time;
    unsigned char color0 = 0;
    unsigned char color1 = 0;
    unsigned char chroma = 0;
    unsigned char fade_divisor = 0;

    wdt_disable();
    init_pwm();
    init_adc();

    TCCR0 = 0x03;

    pwm0 = 0;
    pwm1 = 0;

    light_change();
    
    while(1) {
        if(light_change() && chroma < 0xF0)
            chroma += 0x10;

        fade_divisor++;
        if(chroma && fade_divisor&1)
            --chroma;

        color0 = 0xFF - chroma;
        color1 = chroma;

        delay_time = 0xFF - (abs(color0 - pwm0) >> 1) - (abs(color1 - pwm1) >> 1);

        if(pwm0 > color0 && pwm0 > MIN_PWM)
	    pwm0--;
	if(pwm0 < color0 && pwm0 < MAX_PWM)
	    pwm0++;    
	if(pwm1 > color1 && pwm1 > MIN_PWM)
	    pwm1--;
	if(pwm1 < color1 && pwm1 < MAX_PWM)
	    pwm1++;

	delay(delay_time);
	delay(delay_time);
    }
}
